set fid 0

set tcp_num 0
set tfrc_num 0
set tear_num 0
set pcc_num 0
set tfmcc_num 0
set tfmcc_recv_num 0
set udp_num 0

proc close-all {} {
    global ns tcp_tracefile tfrc_tracefile tear_tracefile pcc_tracefile tfmcc_tracefile cbr_tracefile
    global tcp_num tfrc_num tfmcc_num cbr_num pcc_num tear_num
    $ns flush-trace

    for {set i 0} {$i < $tcp_num} {incr i} {
	if {[info exists tcp_tracefile($i)]} {
	    close $tcp_tracefile($i)
	}
    }
    for {set i 0} {$i < $tfrc_num} {incr i} {
	if {[info exists tfrc_tracefile($i)]} {
	    close $tfrc_tracefile($i)
	}
    }
    for {set i 0} {$i < $tear_num} {incr i} {
	if {[info exists tear_tracefile($i)]} {
	    close $tear_tracefile($i)
	}
    }
    for {set i 0} {$i < $pcc_num} {incr i} {
	if {[info exists pcc_tracefile($i)]} {
	    close $pcc_tracefile($i)
	}
    }
    for {set i 0} {$i < $tfmcc_num} {incr i} {
	if {[info exists tfmcc_tracefile($i)]} {
	    close $tfmcc_tracefile($i)
	}
    }
}

proc weight-string { weights } {
    set delta [expr 1.0 / (($weights + 1) / 2 + 1)];
    set ws [expr $weights+1]+1.0
    for {set i 0} {$i < [expr $weights / 2]} {incr i} {
	set ws "$ws+1.0"
    }
    set w_tmp 1.0
    for {set i [expr $weights / 2]} {$i < $weights} {incr i} {
	set w_tmp [expr $w_tmp - $delta]
	set ws "$ws+$w_tmp"
    }
    return $ws
}

proc minrecvDump { interval file } {
    global rdump_inst_ ns min_nr tfmcc_recv_num tfmcc_sink
    if ![info exists rdump_inst_] {
	set rdump_inst_ 1
	set min_nr 0
	$ns at 0.0 "minrecvDump $interval $file"
	return
    }
    $ns at [expr [$ns now] + $interval] "minrecvDump $interval $file"

    set min_rate [$tfmcc_sink($min_nr) set rate_]
    set min_i -1
    for {set i 0} {$i < $tfmcc_recv_num} {incr i} {
	set tmp_rate [$tfmcc_sink($i) set rate_]
	if {$tmp_rate < $min_rate} {
	    set min_rate $tmp_rate
	    set min_i $i
	}
    }
    set report "[$ns now] $min_rate $min_i"
    puts $file $report
}

proc tfmccDump { src interval file } {
    global dump_inst_ ns
    if ![info exists dump_inst_($src)] {
	set dump_inst_($src) 1
	$ns at 0.0 "tfmccDump $src $interval $file"
	return
    }
    $ns at [expr [$ns now] + $interval] "tfmccDump $src $interval $file"
    set rate [expr ([$src set rate_] * 8.0 / 1000.0)]
    set report "[$ns now] $rate [$src set ndatapack_]"
    puts $file $report
}

proc tfrcDump { src interval file } {
    global dump_inst_ ns tfrc_prev
    if ![info exists dump_inst_($src)] {
	set dump_inst_($src) 1
	set tfrc_prev($src) 0
	$ns at 0.0 "tfrcDump $src $interval $file"
	return
    }
    $ns at [expr [$ns now] + $interval] "tfrcDump $src $interval $file"
    set rate [expr ([$src set ndatapack_] - $tfrc_prev($src)) * 8.0 * [$src set packetSize_] / 1000.0 / $interval]
    if {$rate < 0} {set rate 0}
    set report "[$ns now] $rate [$src set ndatapack_]"
    puts $file $report
    set tfrc_prev($src) [$src set ndatapack_]
}

proc tearDump { src interval file } {
    global dump_inst_ ns tear_prev
    if ![info exists dump_inst_($src)] {
	set dump_inst_($src) 1
	set tear_prev($src) 0
	$ns at 0.0 "tearDump $src $interval $file"
	return
    }
    $ns at [expr [$ns now] + $interval] "tearDump $src $interval $file"
    set rate [expr ([$src set ndatapack_] - $tear_prev($src)) * 8.0 * [$src set packetSize_] / 1000.0 / $interval]
    if {$rate < 0} {set rate 0}
    set report "[$ns now] $rate [$src set ndatapack_]"
    puts $file $report
    set tear_prev($src) [$src set ndatapack_]
}

proc pccDump { src interval file } {
    global dump_inst_ ns pcc_prev
    if ![info exists dump_inst_($src)] {
	set dump_inst_($src) 1
	set pcc_prev($src) 0
	$ns at 0.0 "pccDump $src $interval $file"
	return
    }
    $ns at [expr [$ns now] + $interval] "pccDump $src $interval $file"
    set rate [expr ([$src set ndatapack_] - $pcc_prev($src)) * 8.0 * [$src set packetSize_] / 1000.0 / $interval]
    if {$rate < 0} {set rate 0}
    set report "[$ns now] $rate [$src set ndatapack_]"
    puts $file $report
    set pcc_prev($src) [$src set ndatapack_]
}

proc tcpDump { src interval file } {
    global dump_inst_ ns tcp_prev
    if ![info exists dump_inst_($src)] {
        set dump_inst_($src) 1
	set tcp_prev($src) 0
        $ns at 0.0 "tcpDump $src $interval $file"
        return
    }
    $ns at [expr [$ns now] + $interval] "tcpDump $src $interval $file"
    set rate [expr ([$src set ndatapack_] - $tcp_prev($src)) * 8.0 * [$src set packetSize_] / 1000.0 / $interval]
    if {$rate < 0} {set rate 0}
    set report "[$ns now] $rate [$src set ndatapack_]"
    puts $file $report
    set tcp_prev($src) [$src set ndatapack_]
}

proc setupTCP {start stop tracename s r} {
    global ns tcp_num fid interval tcp tcp_sink ftp tcp_tracefile

    set tcp($tcp_num) [new Agent/TCP/Sack1]
    $tcp($tcp_num) set fid_ $fid
    set tcp_sink($tcp_num) [new Agent/TCPSink/Sack1]
    $ns attach-agent $s $tcp($tcp_num)
    $ns attach-agent $r $tcp_sink($tcp_num)
    $ns connect $tcp($tcp_num) $tcp_sink($tcp_num)
    set ftp($tcp_num) [new Application/FTP]
    $ftp($tcp_num) attach-agent $tcp($tcp_num)
    $ns at $start "$ftp($tcp_num) start"
    $ns at $stop "$ftp($tcp_num) stop"
    if {$tracename != ""} {
	set tcp_tracefile($tcp_num) [open $tracename w]
	tcpDump $tcp($tcp_num) $interval $tcp_tracefile($tcp_num)
    }
    incr tcp_num
    incr fid
    return $tcp([expr $tcp_num-1])
}

proc setupTFRC {start stop tracename s r} {
    global ns tfrc_num fid interval tfrc tfrc_sink tfrc_tracefile

    set tfrc($tfrc_num) [new Agent/TFRC]
    $tfrc($tfrc_num) set fid_ $fid
    set tfrc_sink($tfrc_num) [new Agent/TFRCSink]
    $ns attach-agent $s $tfrc($tfrc_num)
    $ns attach-agent $r $tfrc_sink($tfrc_num)
    $ns connect $tfrc($tfrc_num) $tfrc_sink($tfrc_num)
    $ns at $start "$tfrc($tfrc_num) start"
    $ns at $stop "$tfrc($tfrc_num) stop"
    if {$tracename != ""} {
	set tfrc_tracefile($tfrc_num) [open $tracename w]
	tfrcDump $tfrc($tfrc_num) $interval $tfrc_tracefile($tfrc_num)
    }
    incr tfrc_num
    incr fid
    return $tfrc([expr $tfrc_num-1])
}

proc setupTEAR {start stop tracename s r} {
    global ns tear_num fid interval tear tear_sink tear_tracefile

    set tear($tear_num) [new Agent/TEAR]
    $tear($tear_num) set fid_ $fid
    set tear_sink($tear_num) [new Agent/TEARSink]
    $ns attach-agent $s $tear($tear_num)
    $ns attach-agent $r $tear_sink($tear_num)
    $ns connect $tear($tear_num) $tear_sink($tear_num)
    $ns at $start "$tear($tear_num) start"
    $ns at $stop "$tear($tear_num) stop"
    if {$tracename != ""} {
	set tear_tracefile($tear_num) [open $tracename w]
	tearDump $tear($tear_num) $interval $tear_tracefile($tear_num)
    }
    incr tear_num
    incr fid
    return $tear([expr $tear_num-1])
}

proc setupPCC {start stop tracename s r} {
    global ns pcc_num fid interval pcc pcc_sink pcc_tracefile

    set pcc($pcc_num) [new Agent/PCC]
    $pcc($pcc_num) set fid_ $fid
    set pcc_sink($pcc_num) [new Agent/PCCSink]
    $ns attach-agent $s $pcc($pcc_num)
    $ns attach-agent $r $pcc_sink($pcc_num)
    $ns connect $pcc($pcc_num) $pcc_sink($pcc_num)
    $ns at $start "$pcc($pcc_num) start"
    $ns at $stop "$pcc($pcc_num) stop"
    if {$tracename != ""} {
	set pcc_tracefile($pcc_num) [open $tracename w]
	pccDump $pcc($pcc_num) $interval $pcc_tracefile($pcc_num)
    }
    incr pcc_num
    incr fid
    return $pcc([expr $pcc_num-1])
}

proc setupTFMCC {start stop tracename s} {
    global ns tfmcc_num fid interval tfmcc tfmcc_tracefile

    set group [Node allocaddr]
    set tfmcc($tfmcc_num) [new Agent/TFMCC]
    $tfmcc($tfmcc_num) set fid_ $fid
    $tfmcc($tfmcc_num) set dst_addr_ $group
    $tfmcc($tfmcc_num) set dst_port_ 0
    $ns attach-agent $s $tfmcc($tfmcc_num)
    $ns at $start "$tfmcc($tfmcc_num) start"
    $ns at $stop "$tfmcc($tfmcc_num) stop"
    if {$tracename != ""} {
	set tfmcc_tracefile($tfmcc_num) [open $tracename w]
	tfmccDump $tfmcc($tfmcc_num) $interval $tfmcc_tracefile($tfmcc_num)
    }
    incr tfmcc_num
    incr fid
    return $tfmcc([expr $tfmcc_num-1])
}

proc setupTFMCCSink {start stop t r} {
    global ns tfmcc_recv_num tfmcc_sink

    set group [$t set dst_addr_]
    set tfmcc_sink($tfmcc_recv_num) [new Agent/TFMCCSink]
    $ns attach-agent $r $tfmcc_sink($tfmcc_recv_num)
    $tfmcc_sink($tfmcc_recv_num) set dst_addr_ [$t set agent_addr_]
    $tfmcc_sink($tfmcc_recv_num) set dst_port_ [$t set agent_port_]
    $tfmcc_sink($tfmcc_recv_num) set recv_id_ $tfmcc_recv_num
    $ns at $start "$r join-group $tfmcc_sink($tfmcc_recv_num) $group"
    $ns at $stop "$tfmcc_sink($tfmcc_recv_num) stop; $r leave-group $tfmcc_sink($tfmcc_recv_num) $group"
    incr tfmcc_recv_num
    return $tfmcc_sink([expr $tfmcc_recv_num-1])
}

proc setupUDP {start stop tracename s r} {
    global ns udp_num fid interval udp cbr null

    set udp($udp_num) [new Agent/UDP]
    $udp($udp_num) set fid_ fid
    $ns attach-agent $s $udp($udp_num)
    set cbr($udp_num) [new Application/Traffic/CBR]
    $cbr($udp_num) attach-agent $udp($udp_num)
    set null($udp_num) [new Agent/Null] 
    $ns attach-agent $r $null($udp_num)
    $ns connect $udp($udp_num) $null($udp_num)
    $ns at $start "$cbr($udp_num) start"
    $ns at $stop "$cbr($udp_num) stop"
    incr udp_num
    incr fid
    return $cbr([expr $udp_num-1])
}

proc setupPareto {start stop tracename s r} {
    global ns udp_num fid interval udp prt null

    set udp($udp_num) [new Agent/UDP]
    $udp($udp_num) set fid_ fid
    $ns attach-agent $s $udp($udp_num)
    set prt($udp_num) [new Application/Traffic/Pareto]
    $prt($udp_num) attach-agent $udp($udp_num)
    set null($udp_num) [new Agent/Null] 
    $ns attach-agent $r $null($udp_num)
    $ns connect $udp($udp_num) $null($udp_num)
    $ns at $start "$prt($udp_num) start"
    $ns at $stop "$prt($udp_num) stop"
    incr udp_num
    incr fid
    return $prt([expr $udp_num-1])
}
